Unit RECUNIT;

Interface

Uses

Windows, MMSystem, MSACM;

//  TWaveRecorder    
//    TWaveRecorder.ProcessBuffer.  
//       -

Const

MAX_BUFFERS = 8;

type

PWaveRecorder = ^TWaveRecorder;
TWaveRecorder = class(TObject)
Constructor Create(BfSize, TotalBuffers : Integer);
Destructor  Destroy;      Override;
Procedure   ProcessBuffer(uMsg : Word; P : Pointer; n : Integer; dwParam1: dword); Virtual;


private                                            
fBufferSize        : Integer; //  
BufIndex           : Integer;
fTotalBuffers      : Integer;


pWaveHeader        : Array [0..MAX_BUFFERS-1] of PWAVEHDR;
hWaveHeader        : Array [0..MAX_BUFFERS-1] of THANDLE;
hWaveBuffer        : Array [0..MAX_BUFFERS-1] of THANDLE;
hWaveFmtEx         : THANDLE;
dwByteDataSize     : DWORD;
dwTotalWaveSize    : DWORD;


RecordActive       : Boolean;
bDeviceOpen        : Boolean;


{    }
Function InitWaveHeaders : Boolean;
Function AllocPCMBuffers : Boolean;
Procedure FreePCMBuffers;


Function AllocWaveFormatEx : Boolean;
Procedure FreeWaveFormatEx;


Function AllocWaveHeaders : Boolean;
Procedure FreeWaveHeader;


Function AddNextBuffer : Boolean;
Procedure CloseWaveDeviceRecord;


public
{ Public declarations }
pWaveFmtEx         : PWaveFormatEx;
WaveBufSize        : Integer;          //   nBlockAlign
InitWaveRecorder   : Boolean;
RecErrorMessage    : String;
QueuedBuffers,
ProcessedBuffers   : Integer;
pWaveBuffer        : Array [0..MAX_BUFFERS-1] of lpstr;
WaveIn             : HWAVEIN; {  Wav- }


Procedure StopRecord;
Function StartRecord : Boolean;
Function SetupRecord(P : PWaveRecorder) : Boolean;

end;

implementation

//TWaveInGetErrorText
Function TWaveInGetErrorText(iErr : Integer) : String;
{     WaveIn   Pascal                  }
{ iErr -                                                    }
Var

PlayInErrorMsgC   : Array [0..255] of Char;

Begin

waveInGetErrorText(iErr,PlayInErrorMsgC,255);
TWaveInGetErrorText := PlayInErrorMsgC;
End;

//InitWaveHeaders
Function TWaveRecorder.AllocWaveFormatEx : Boolean;
{    ,    ACM-}
Var MaxFmtSize : UINT;
BEGIN
{ maxFmtSize -  sizeof(WAVEFORMATEX) + pwavefmtex.cbSize }
If( acmMetrics( 0, ACM_METRIC_MAX_SIZE_FORMAT, maxFmtSize ) <> 0) Then
Begin
RecErrorMessage := '     ';
AllocWaveFormatEx := False;
Exit;
End;

{   WAVEFMTEX }
hWaveFmtEx := GlobalAlloc(GMEM_MOVEABLE, maxFmtSize);
If (hWaveFmtEx = 0) Then
Begin
RecErrorMessage := '     WaveFormatEx';
AllocWaveFormatEx := False;
Exit;
End;

pWaveFmtEx := PWaveFormatEx(GlobalLock(hWaveFmtEx));
If (pWaveFmtEx = Nil) Then
Begin
RecErrorMessage := '   WaveFormatEx';
AllocWaveFormatEx := False;
Exit;
End;

{     PCM }
ZeroMemory( pwavefmtex, maxFmtSize );
pwavefmtex.wFormatTag := WAVE_FORMAT_PCM;
pwavefmtex.nChannels := 1;
pwavefmtex.nSamplesPerSec := 20000;
pwavefmtex.nBlockAlign := 1;
pwavefmtex.wBitsPerSample := 16;
pwavefmtex.nAvgBytesPerSec := pwavefmtex.nSamplesPerSec*
(pwavefmtex.wBitsPerSample div 8)*pwavefmtex.nChannels;
pwavefmtex.cbSize := 0;

AllocWaveFormatEx := True;
end;

//InitWaveHeaders
Function TWaveRecorder.InitWaveHeaders : Boolean;

{  ,   wave          }
Var i : Integer;
BEGIN
{      ... }
WaveBufSize := fBufferSize - (fBufferSize mod pwavefmtex.nBlockAlign);


{  wave- }
For i := 0 to fTotalBuffers-1 Do
With pWaveHeader[i]^ Do
Begin
lpData := pWaveBuffer[i];      //   waveform
dwBufferLength := WaveBufSize; // ,  , 
dwBytesRecorded := 0;          //  
dwUser := 0;                   // 32   
dwFlags := 0;                  //  
dwLoops := 0;                  //  
lpNext := Nil;                 // ;   
reserved := 0;                 // ;   
End;

InitWaveHeaders := TRUE;
END;

//AllocWaveHeader
Function TWaveRecorder.AllocWaveHeaders : Boolean;
{                                  }
Var i : Integer;
BEGIN

For i := 0 to fTotalBuffers-1 Do
begin
hwaveheader[i] := GlobalAlloc( GMEM_MOVEABLE or GMEM_SHARE or
GMEM_ZEROINIT, sizeof(TWAVEHDR));

if (hwaveheader[i] = 0) Then
begin
{ :      ,    }
RecErrorMessage := '    wave-';
AllocWaveHeaders := FALSE;
Exit;
end;


pwaveheader[i] := GlobalLock (hwaveheader[i]);
If (pwaveheader[i] = Nil ) Then
begin
{ :      ,    }
RecErrorMessage := '      ';
AllocWaveHeaders := FALSE;
Exit;
end;

End;

AllocWaveHeaders := TRUE;
END;

//FreeWaveHeader-
Procedure TWaveRecorder.FreeWaveHeader;
{    AllocWaveHeaders .            }
Var i : Integer;
BEGIN

For i := 0 to fTotalBuffers-1 Do
begin
If (hWaveHeader[i] <> 0) Then
Begin
GlobalUnlock(hwaveheader[i]);
GlobalFree(hwaveheader[i]);
hWaveHeader[i] := 0;
End
end;
END;


//AllocPCMBuffers
Function TWaveRecorder.AllocPCMBuffers : Boolean;
{     waveform.                             }
Var i : Integer;
BEGIN

For i := 0 to fTotalBuffers-1 Do
begin
hWaveBuffer[i] := GlobalAlloc( GMEM_MOVEABLE or GMEM_SHARE, fBufferSize );
If (hWaveBuffer[i] = 0) Then
begin
{     }
RecErrorMessage := '   wave-';
AllocPCMBuffers := False;
Exit;
end;

pWaveBuffer[i] := GlobalLock(hWaveBuffer[i]);
If (pWaveBuffer[i] = Nil) Then
begin
{     }
RecErrorMessage := '   wave-';
AllocPCMBuffers := False;
Exit;
end;
pWaveHeader[i].lpData := pWaveBuffer[i];
End;

AllocPCMBuffers := TRUE;
END;

//FreePCMBuffers
Procedure TWaveRecorder.FreePCMBuffers;
{   AllocPCMBuffers .                    }
Var i : Integer;
BEGIN

For i := 0 to fTotalBuffers-1 Do
begin
If (hWaveBuffer[i] <> 0) Then
Begin
GlobalUnlock( hWaveBuffer[i] );
GlobalFree( hWaveBuffer[i] );
hWaveBuffer[i] := 0;
pWaveBuffer[i] := Nil;
End;
end;
END;

//FreeWaveFormatEx
Procedure TWaveRecorder.FreeWaveFormatEx;
{    ExFormat headers                         }
BEGIN

If (pWaveFmtEx = Nil) Then Exit;
GlobalUnlock(hWaveFmtEx);
GlobalFree(hWaveFmtEx);
pWaveFmtEx := Nil;
END;

//TWaveRecorder.Create
Constructor TWaveRecorder.Create(BFSize, TotalBuffers : Integer);
{  wave-,          }
{                                      }
{ BFSize -                                          }

Var i : Integer;
BEGIN

Inherited Create;
For i := 0 to fTotalBuffers-1 Do
Begin
hWaveHeader[i] := 0;
hWaveBuffer[i] := 0;
pWaveBuffer[i] := Nil;
pWaveFmtEx := Nil;
End;
fBufferSize := BFSize;


fTotalBuffers := TotalBuffers;
{     wave- }
If(Not AllocWaveFormatEx) Then
Begin
InitWaveRecorder := FALSE;
Exit;
End;

{  ,    wave- }
If (waveInGetNumDevs < 1 ) Then
Begin
RecErrorMessage := '  ,   ';
InitWaveRecorder := FALSE;
Exit;
End;

{   wave- }
If (Not AllocWaveHeaders) Then
Begin
InitWaveRecorder := FALSE;
Exit;
End;

{    wave- }
If (Not AllocPCMBuffers)  Then
Begin
InitWaveRecorder := FALSE;
Exit;
End;

InitWaveRecorder := TRUE;

END;

//Destroy-
Destructor TWaveRecorder.Destroy;
{    ,  InitWaveRecorder.       }
BEGIN

FreeWaveFormatEx;
FreePCMBuffers;
FreeWaveHeader;
Inherited Destroy;
END;

//CloseWaveDeviceRecord
Procedure TWaveRecorder.CloseWaveDeviceRecord;
{   () waveform-.                   }
Var i : Integer;
BEGIN

{    ,   }
If (Not bDeviceOpen) Then Exit;


{    - unprepare }
For i := 0 to fTotalBuffers-1 Do
If (waveInUnprepareHeader(WaveIn, pWaveHeader[i], sizeof(TWAVEHDR)) <> 0 )
Then

RecErrorMessage := '  waveInUnprepareHeader';

{        }
dwTotalwavesize := dwBytedatasize;

{   wave- }
If (waveInClose(WaveIn) <> 0) Then
RecErrorMessage := '   ';

{   ,    }
bDeviceOpen := FALSE;

END;

//StopRecord
Procedure TWaveRecorder.StopRecord;
{      .                 }
Var iErr : Integer;
BEGIN

RecordActive := False;
iErr := waveInReset(WaveIn);
{         }
If (iErr <> 0) Then
Begin
RecErrorMessage := '  waveInReset';
End;

CloseWaveDeviceRecord;
END;

//AddNextBuffer
Function TWaveRecorder.AddNextBuffer : Boolean;
{         .     }
Var iErr : Integer;
BEGIN

{          }
iErr := waveInAddBuffer(WaveIn, pwaveheader[bufindex], sizeof(TWAVEHDR));
If (iErr <> 0) Then
begin
StopRecord;
RecErrorMessage := '  ' + TWaveInGetErrorText(iErr);
AddNextBuffer := FALSE;
Exit;
end;

{     }
bufindex := (bufindex+1) mod fTotalBuffers;
QueuedBuffers := QueuedBuffers + 1;

AddNextBuffer := TRUE;
END;

//BufferDoneCallBack
Procedure BufferDoneCallBack(
hW    : HWAVE;      //  waveform-
uMsg  : DWORD;      //  
dwInstance : DWORD; //  
dwParam1 : DWORD;   //   
dwParam2 : DWORD   //   
);  stdcall;

{     wave- - ,       }
Var BaseRecorder : PWaveRecorder;
BEGIN

BaseRecorder := Pointer(DwInstance);
With BaseRecorder^ Do
Begin
ProcessBuffer(uMsg, pWaveBuffer[ProcessedBuffers Mod fTotalBuffers],
WaveBufSize, dwParam1);

If (RecordActive) Then
Case uMsg of
WIM_DATA:
Begin
BaseRecorder.AddNextBuffer;
ProcessedBuffers := ProcessedBuffers+1;
End;
End;
End;
END;

//StartRecord
Function TWaveRecorder.StartRecord : Boolean;
{  .                                                        }
Var iErr, i : Integer;
BEGIN

{      }
iErr := WaveInStart(WaveIn);
If (iErr <> 0) Then
begin
 CloseWaveDeviceRecord;
 RecErrorMessage := '   wave: ' +
 TWaveInGetErrorText(iErr);
end;

 RecordActive := TRUE;

{      }
For i := 1 to fTotalBuffers-1 Do
If (Not AddNextBuffer) Then
Begin
StartRecord := FALSE;
Exit;
End;

StartRecord := True;
END;

//SetupRecord
Function TWaveRecorder.SetupRecord(P : PWaveRecorder) : Boolean;
{        waveform-"". }
Var iErr, i : Integer;
BEGIN

dwTotalwavesize := 0;
dwBytedatasize := 0;
bufindex := 0;
ProcessedBuffers := 0;
QueuedBuffers := 0;

{     }
iErr := waveInOpen(@WaveIn, WAVE_MAPPER, pWaveFmtEx,
Integer(@BufferDoneCallBack),

Integer(P), CALLBACK_FUNCTION + WAVE_ALLOWSYNC );
If (iErr <> 0) Then
Begin
RecErrorMessage := '      : ' + ^M 
+ TWaveInGetErrorText(iErr);
SetupRecord := FALSE;
Exit;
End;

// CloseWaveDeviceRecord(),   
bDeviceOpen := TRUE;

// 
InitWaveHeaders();

For i := 0 to fTotalBuffers-1 Do
Begin
iErr := waveInPrepareHeader( WaveIn, pWaveHeader[I], sizeof(TWAVEHDR));
If (iErr <> 0) Then
begin
CloseWaveDeviceRecord;
RecErrorMessage := '    : ' + ^M +
TWaveInGetErrorText(iErr);
SetupRecord := FALSE;
Exit;
end;
End;

//  
If (Not AddNextBuffer) Then
begin
SetupRecord := FALSE;
Exit;
end;

SetupRecord := TRUE;
END;

//ProcessBuffer
Procedure TWaveRecorder.ProcessBuffer(uMsg: Word; P : Pointer; n :Integer; dwParam1: dword);
BEGIN
END;

END.
